@@ -1,8 +1,9 @@
 package main
 
 import (
-	. "./example"
 	"fmt"
+
+	. "example"
 )
 
 func main() {
