@@ -1,6 +1,6 @@
 package main
 
-import dc "./default_constructor"
+import dc "default_constructor"
 
 func main() {
 	a := dc.NewA()
@@ -21,9 +21,6 @@ func main() {
 	f := dc.NewF()
 	f.Destroy()
 
-	ff := dc.NewFFF()
-	ff.Destroy()
-
 	g := dc.NewG()
 
 	dc.GDestroy(g)
