@@ -1,6 +1,6 @@
 package main
 
-import "./typemap_ns_using"
+import "typemap_ns_using"
 
 func main() {
 	if typemap_ns_using.Spam(37) != 37 {
