@@ -33,6 +33,7 @@ static bool js_template_enable_debug = false;
 #define GETTER "getter"
 #define SETTER "setter"
 #define PARENT "parent"
+#define PARENT_MANGLED "parent_mangled"
 #define CTOR "ctor"
 #define CTOR_WRAPPERS "ctor_wrappers"
 #define CTOR_DISPATCHERS "ctor_dispatchers"
@@ -262,7 +263,7 @@ protected:
 
   virtual int createNamespace(String *scope);
 
-  virtual Hash *createNamespaceEntry(const char *name, const char *parent);
+  virtual Hash *createNamespaceEntry(const char *name, const char *parent, const char *parent_mangled);
 
   virtual int emitNamespaces() = 0;
 
@@ -462,10 +463,10 @@ int JAVASCRIPT::fragmentDirective(Node *n) {
   // and register them at the emitter.
   String *section = Getattr(n, "section");
 
-  if (Equal(section, "templates")) {
+  if (Equal(section, "templates") && !ImportMode) {
     emitter->registerTemplate(Getattr(n, "value"), Getattr(n, "code"));
   } else {
-    Swig_fragment_register(n);
+    return Language::fragmentDirective(n);
   }
 
   return SWIG_OK;
@@ -552,7 +553,7 @@ void JAVASCRIPT::main(int argc, char *argv[]) {
       emitter = swig_javascript_create_V8Emitter();
       Preprocessor_define("SWIG_JAVASCRIPT_V8 1", 0);
       SWIG_library_directory("javascript/v8");
-      // V8 API is C++, so output must be C++ compatibile even when wrapping C code
+      // V8 API is C++, so output must be C++ compatible even when wrapping C code
       if (!cparse_cplusplus) {
 	Swig_cparse_cplusplusout(1);
       }
@@ -664,7 +665,7 @@ int JSEmitter::initialize(Node * /*n */ ) {
     Delete(namespaces);
   }
   namespaces = NewHash();
-  Hash *global_namespace = createNamespaceEntry("exports", 0);
+  Hash *global_namespace = createNamespaceEntry("exports", 0, 0);
 
   Setattr(namespaces, "::", global_namespace);
   current_namespace = global_namespace;
@@ -993,7 +994,7 @@ int JSEmitter::emitDtor(Node *n) {
 
      Also, there is a problem where destructor_action is always true for me, even when not requesting %extend as above.
      So this code doesn't actually quite work as I expect. The end result is that the code still works because
-     destructor_action calls free like the original template. The one caveat is the string in destructor_action casts to char* which is wierd.
+     destructor_action calls free like the original template. The one caveat is the string in destructor_action casts to char* which is weird.
      I think there is a deeper underlying SWIG issue because I don't think it should be char*. However, it doesn't really matter for free.
 
      Maybe the fix for the destructor_action always true problem is that this is supposed to be embedded in the if(Extend) block above.
@@ -1124,7 +1125,7 @@ int JSEmitter::emitConstant(Node *n) {
   Template t_getter(getTemplate("js_getter"));
 
   // call the variable methods as a constants are
-  // registred in same way
+  // registered in same way
   enterVariable(n);
   state.variable(GETTER, wname);
   // TODO: why do we need this?
@@ -1428,19 +1429,20 @@ int JSEmitter::createNamespace(String *scope) {
   }
   assert(parent_namespace != 0);
 
-  Hash *new_namespace = createNamespaceEntry(Char(scope), Char(Getattr(parent_namespace, "name")));
+  Hash *new_namespace = createNamespaceEntry(Char(scope), Char(Getattr(parent_namespace, "name")), Char(Getattr(parent_namespace, "name_mangled")));
   Setattr(namespaces, scope, new_namespace);
 
   Delete(parent_scope);
   return SWIG_OK;
 }
 
-Hash *JSEmitter::createNamespaceEntry(const char *_name, const char *parent) {
+Hash *JSEmitter::createNamespaceEntry(const char *_name, const char *parent, const char *parent_mangled) {
   Hash *entry = NewHash();
   String *name = NewString(_name);
   Setattr(entry, NAME, Swig_scopename_last(name));
   Setattr(entry, NAME_MANGLED, Swig_name_mangle(name));
   Setattr(entry, PARENT, NewString(parent));
+  Setattr(entry, PARENT_MANGLED, NewString(parent_mangled));
 
   Delete(name);
   return entry;
@@ -1467,7 +1469,7 @@ protected:
   virtual int enterClass(Node *n);
   virtual int exitClass(Node *n);
   virtual void marshalInputArgs(Node *n, ParmList *parms, Wrapper *wrapper, MarshallingMode mode, bool is_member, bool is_static);
-  virtual Hash *createNamespaceEntry(const char *name, const char *parent);
+  virtual Hash *createNamespaceEntry(const char *name, const char *parent, const char *parent_mangled);
   virtual int emitNamespaces();
 
 private:
@@ -1773,8 +1775,8 @@ int JSCEmitter::exitClass(Node *n) {
   return SWIG_OK;
 }
 
-Hash *JSCEmitter::createNamespaceEntry(const char *name, const char *parent) {
-  Hash *entry = JSEmitter::createNamespaceEntry(name, parent);
+Hash *JSCEmitter::createNamespaceEntry(const char *name, const char *parent, const char *parent_mangled) {
+  Hash *entry = JSEmitter::createNamespaceEntry(name, parent, parent_mangled);
   Setattr(entry, "functions", NewString(""));
   Setattr(entry, "values", NewString(""));
   return entry;
@@ -1786,8 +1788,7 @@ int JSCEmitter::emitNamespaces() {
     Hash *entry = it.item;
     String *name = Getattr(entry, NAME);
     String *name_mangled = Getattr(entry, NAME_MANGLED);
-    String *parent = Getattr(entry, PARENT);
-    String *parent_mangled = Swig_name_mangle(parent);
+    String *parent_mangled = Getattr(entry, PARENT_MANGLED);
     String *functions = Getattr(entry, "functions");
     String *variables = Getattr(entry, "values");
 
@@ -2037,7 +2038,7 @@ int V8Emitter::exitClass(Node *n) {
 	.pretty_print(f_init_inheritance);
     Delete(base_name_mangled);
   }
-  //  emit registeration of class template
+  //  emit registration of class template
   Template t_register = getTemplate("jsv8_register_class");
   t_register.replace("$jsmangledname", state.clazz(NAME_MANGLED))
       .replace("$jsname", state.clazz(NAME))
@@ -2206,7 +2207,7 @@ int V8Emitter::emitNamespaces() {
     String *name = Getattr(entry, NAME);
     String *name_mangled = Getattr(entry, NAME_MANGLED);
     String *parent = Getattr(entry, PARENT);
-    String *parent_mangled = Swig_name_mangle(parent);
+    String *parent_mangled = Getattr(entry, PARENT_MANGLED);
 
     bool do_create = true;
     bool do_register = true;
@@ -2427,7 +2428,7 @@ Template & Template::trim() {
 /* -----------------------------------------------------------------------------
  * Template&  Template::replace(const String* pattern, const String* repl) :
  *
- *  replaces all occurences of a given pattern with a given replacement.
+ *  replaces all occurrences of a given pattern with a given replacement.
  *
  *  - pattern:  the pattern to be replaced
  *  - repl:     the replacement string
